define Class aBaseObject as Custom
  *
  * Member variables
  * ----------------
  *
  nReturnCode = 0
  cReturnKey  = [ 'PROCESS_SUCCESS_LOC' ]
  *
  oParameter   = null
  *
  * Public Interface
  * ----------------
  *
  procedure Init
  *===========================================================================
  * Method.....: Init
  * Author.....: Mark Austen
  * Date.......: February 25, 2002
  * Notice.....: Copyright ( c ) 2002 Schooner Software Limited,
  * ...........: All Rights Reserved.
  * Abstract...: Setup the parameters for This object.
  *===========================================================================
  *
  lparameters ;
    poParameter
  *
  with This
    *
    if file( 'logobject.txt' )
      .Tag = sys( 2015 )
      do LogObject with .Tag, .Name, .Class, .t.
      *
    endif &&  && file( 'logobject.txt' )
    *
    .Setup( poParameter )
    *
  endwith &&  && This
  *
  return
  *
  *===========================================================================
  * End: Init
  *===========================================================================
  endproc && Init
  *
  procedure Destroy
  *===========================================================================
  * Method.....: Destroy
  * Author.....: Mark Austen
  * Date.......: April 4, 2002
  * Notice.....: Copyright ( c ) 2002 Schooner Software Limited,
  * ...........: All Rights Reserved.
  * Abstract...: Null any objects
  *===========================================================================
  *
  with This
    *
    if IsObject( .oParameter )
      .oParameter = null
      *
    endif &&  && IsObject( poParameter )
    *
    if file( 'logobject.txt' )
      do LogObject with .Tag, .Name, .Class, .f.
      *
    endif && file( 'logobject.txt' )
    *
  endwith && This
  *
  return
  *
  *===========================================================================
  * End: Destroy
  *===========================================================================
  endproc && Destroy
  *
  procedure Release
  *===========================================================================
  * Method.....: Release
  * Author.....: Mark Austen
  * Date.......: February 25, 2002
  * Notice.....: Copyright ( c ) 2002 Schooner Software Limited,
  * ...........: All Rights Reserved.
  * Abstract...: To bring Class inline with VFP native Release method.
  *===========================================================================
  *
  Release This
  *
  return
  *
  *===========================================================================
  * End: Release
  *===========================================================================
  endproc
  *
  procedure Process
  *===========================================================================
  * Method.....: Process
  * Author.....: Mark Austen
  * Date.......: December 18, 2001
  * Notice.....: Copyright ( c ) 2001 Schooner Software Limited,
  * ...........: All Rights Reserved.
  * Abstract...: Process the action - Abstract method
  *===========================================================================
  *
  *===========================================================================
  * End: Process
  *===========================================================================
  endproc
  *
  * Protected Methods
  * -----------------
  *
  procedure Setup
  *===========================================================================
  * Method.....: Setup
  * Author.....: Mark Austen
  * Date.......: February 25, 2002
  * Notice.....: Copyright ( c ) 2002 Schooner Software Limited,
  * ...........: All Rights Reserved.
  * Abstract...: Setup the parameters for This object.
  *===========================================================================
  *
  lparameters ;
    poParameter
  *
  with This
    *
    if IsObject( poParameter )
      .oParameter = poParameter
      *
    endif && IsObject( poParameter )
    *
  endwith && This
  *
  return
  *
  *===========================================================================
  * End: Setup
  *===========================================================================
  endproc && Setup
  *
enddefine &&  && aBaseObject
*
define Class aSDObject as aBaseObject
  *
  * Member variables
  * ----------------
  *
  protected ;
    lQuiet
  *
  lQuiet = .f.
  *
  * Public Interface
  * ----------------
  *
  * Protected Methods
  * -----------------
  *
  procedure Setup
  *===========================================================================
  * Method.....: Setup
  * Author.....: Mark Austen
  * Date.......: February 25, 2002
  * Notice.....: Copyright ( c ) 2002 Schooner Software Limited,
  * ...........: All Rights Reserved.
  * Abstract...: Setup the parameters for This object.
  *===========================================================================
  *
  lparameters ;
    poParameter
  *
  local ;
    lcClass, ;
    QQ___QQ
  *
  dodefault( poParameter )
  *
  with This
    *
    if IsObject( .oParameter )
      .lQuiet = .oParameter.lQuiet
      *
    endif && IsObject( .oParameter )
    *
  endwith && This
  *
  return
  *
  *===========================================================================
  * End: Setup
  *===========================================================================
  endproc && Setup
  *
  procedure ShowProgress
  *===========================================================================
  * Method.....: ShowProgress
  * Author.....: Mark Austen
  * Date.......: February 25, 2002
  * Notice.....: Copyright (c) 2002 Schooner Software Limited,
  * ...........: All Rights Reserved.
  * Abstract...: Dislplay the progress of the Process if required.
  *===========================================================================
  *
  lparameters ;
    pcMessage as string
  *
  local ;
    lcAction, ;
    QQ___QQ
  *
  with This
    *
    if not .lQuiet
      *
      if pemstatus( .oparameter.oParent, 'ShowProgress', 5 )
        *
        with .oParameter.oParent
          .ShowProgress( pcMessage )
          *
        endwith && .oParameter.oParent
        *
      endif && pemstatus( .oparameter.oParent, 'ShowProgress', 5 )
      *
    endif && not .lQuiet
    *
  endwith && This
  *
  return
  *
  *===========================================================================
  * End: ShowProgress
  *===========================================================================
  endproc && ShowProgress
  *
enddefine &&  && aFOPObject
*
define Class ProcessFolders as aSDObject
  *
  * Member variables
  * ----------------
  *
  protected ;
    oMD5
  *
  oMD5 = null
  *
  * Public Interface
  * ----------------
  *
  #define FILE_NAME          1
  #define FILE_SIZE          2
  #define DATE_LAST_MODIFIED 3
  #define TIME_LAST_MODIFIED 4
  #define FILE_ATTRIBUTES    5
  *
  procedure Destroy
  *===========================================================================
  * Method.....: Destroy
  * Author.....: Mark Austen
  * Date.......: April 4, 2002
  * Notice.....: Copyright ( c ) 2002 Schooner Software Limited,
  * ...........: All Rights Reserved.
  * Abstract...: Null any objects
  *===========================================================================
  *
  This.oMD5.Release()
  *
  dodefault()
  *
  return
  *
  *===========================================================================
  * End: Destroy
  *===========================================================================
  endproc && Destroy
  *
  procedure Process
  *===========================================================================
  * Method.....: Process
  * Author.....: Mark Austen
  * Date.......: December 26, 2001
  * Notice.....: Copyright (c) 2001 Schooner Software Limited,
  * ...........: All Rights Reserved.
  * Abstract...: Print the supplied string.
  *===========================================================================
  *
  local ;
    lnFoldersUnprocessed, ;
    lnFolders, ;
    lcFolder, ;
    lcPath, ;
    lcSourcePath, ;
    lnI, ;
    lcSourceRoot, ;
    QQ___QQ
  *
  local array ;
    laFolders[ 1 ]
  *
  lcSourceRoot = lower( addbs( .oParameter.cSourcePath ))
  *
  *** Create a cursor to hold the files and hashes
  *
  lnHashLength = len( This.oMD5.cDigest )
  *
  create ;
    cursor ;
      'curFiles' ;
      ( ;
        cFolder c( 254 ), ;
        cFile   c( 254 ), ;
        nSize   i, ;
        cHash   c( lnHashLength ) ;
      )
  *
  index on cHash tag cHash
  *
  *** Create a cursor to hold the folders
  *
  create ;
    cursor ;
      'curFolders' ;
      ( ;
        cFolder    c( 254 ), ;
        cProcessed c( 1 ) ;
      )
  *
  *** Add a record for the root folder of the source drive to start the ball
  *** rolling
  *
  insert ;
    into ;
      'curFolders' ;
    values ;
    ( ;
      lcSourceRoot, ;
      'N' ;
    )
  *
  lnFoldersUnprocessed = 1
  *
  *** Work through the folders and create a list of all the folders found
  *
  do LogAction with 'Process Started', .t.
  *
  This.ShowProgress( 'Creating Folder List...' )
  *
  do while lnFoldersUnprocessed # 0
    locate ;
      for ;
        cProcessed = 'N'
    *
    replace ;
      cProcessed with 'Y'
    *
    lcPath    = addbs( alltrim( cFolder ))
    lnFolders = adir( laFolders, lcPath + '*.*', 'D' )
    *
    if lnFolders > 0
      *
      for lnI = 1 to lnFolders
        *
        if 'D' $ laFolders[ lnI, 5 ]
          lcFolder = alltrim( laFolders[ lnI, 1 ] )
          *
          if not inlist( lcFolder, '.', '..' )
            insert ;
              into ;
                'curFolders' ;
              values ;
              ( ;
                lower( lcPath + lcFolder ), ;
                'N' ;
              )
            *
          endif && inlist( lcFolder, '.', '..' )
          *
        endif && 'D' $ laFolders[ lnI, 5 ]
        *
      next && lnI
      *
    endif && lnFolders > 0
    *
    count to lnFoldersUnprocessed for cProcessed = 'N'
    *
  enddo && while lnFoldersUnprocessed # 0
  *
  scan for cProcessed == 'Y'
    lcSourcePath = addbs( alltrim( cFolder ) )
    *
    .MakeHashes( lcSourcePath )
    *
  endscan && for cProcessed == 'Y'
  *
  use in 'curFolders'
  *
  select ;
      * ;
    from ;
      'curFiles' ;
    where ;
      cHash in ;
      ( ;
        select ;
            cHash ;
          from ;
            'curFiles' ;
          group by ;
            cHash ;
          having ;
            count( * ) > 1 ;
      ) ;
    into ;
      table 'curDuplicates' ;
    order by ;
      cHash
  *
  use in 'curFiles'
  *
  return
  *
  *===========================================================================
  * End: Process
  *===========================================================================
  endproc && Process
  *
  procedure MakeHashes
  *===========================================================================
  * Procedure..: MakeHashes
  * Author.....: Mark Austen
  * Date.......: October 28, 2002 
  * Notice.....: Copyright (c) 2002 Schooner Software Limited,
  * ...........: All Rights Reserved.
  * Abstract...: get the hases for all the files.
  *===========================================================================
  *
  lparameters ;
    pcSourcePath as string
  *
  local ;
    lnFiles, ;
    lcSourceFile, ;
    lcFile, ;
    lcMD5, ;
    lnSize, ;
    QQ___QQ
  *
  local array ;
    laSourceFiles[ 1 ], ;
    laFile[ 1 ]
  *
  lnFiles = adir( laSourceFiles, pcSourcePath + '*.*' )
  *
  if lnFiles > 0
    *
    for lnI = 1 to lnFiles
      lcMD5        = ''
      lcFile       = lower( alltrim( laSourceFiles[ lnI, 1 ] ) )
      lcSourceFile = pcSourcePath + lcFile
      *
      with This
        .ShowProgress( lcSourceFile )
        lcMD5 = .oMD5.ProcessFile( lcSourceFile )
        *
      endwith && This
      *
      if not empty( lcMD5 )
        adir( laFile, lcSourceFile )
        lnSize = laFile[ 1, 2 ]
        *
        insert ;
          into ;
            'curFiles' ;
            values ;
            ( ;
              pcSourcePath, ;
              lcFile, ;
              lnSize, ;
              lcMD5 ;
            )
        *
      endif && not empty( lcMD5 )
      *
    next && lnI
    *
  endif && lnFiles > 0
  *
  return
  *
  *===========================================================================
  * End: MakeHashes
  *===========================================================================
  endproc && MakeHashes
  *
  procedure Setup
  *===========================================================================
  * Method.....: Setup
  * Author.....: Mark Austen
  * Date.......: February 25, 2002
  * Notice.....: Copyright (c) 2002 Schooner Software Limited,
  * ...........: All Rights Reserved.
  * Abstract...: Setup the parameters for This object.
  *===========================================================================
  *
  lparameters ;
    poParameter
  *
  local ;
    QQ___QQ
  *
  This.oMD5 = createobject( 'MakeHash', poParameter )
  *
  return dodefault( poParameter )
  *
  *===========================================================================
  * End: Setup
  *===========================================================================
  endproc && Setup
  *
enddefine && ProcessFolders
*
define Class MakeHash as aSDObject
  *
  * Member variables
  * ----------------
  *
  cDigest         = ''
  cAlgorithmList  = ''
  cFormatTypeList = ''
  *
  * Public Interface
  * ----------------
  *
  procedure Destroy
  *===========================================================================
  * Method.....: Destroy
  * Author.....: Mark Austen
  * Date.......: April 4, 2002
  * Notice.....: Copyright ( c ) 2002 Schooner Software Limited,
  * ...........: All Rights Reserved.
  * Abstract...: Null any objects
  *===========================================================================
  *
  ResetHash()
  *
  clear dlls ;
    SetHash, ;
    ResetHash, ;
    GetAlgorithmList, ;
    GetAlgorithmBufferSize, ;
    GetFormatList, ;
    GetFormatBufferSize, ;
    SetAlgorithm, ;
    SetFormatType, ;
    GetFormatType, ;
    GetAlgorithm, ;
    GetDigestSize, ;
    CalculateFile, ;
    CalculateString
  *
  dodefault()
  *
  return
  *
  *===========================================================================
  * End: Destroy
  *===========================================================================
  endproc && Destroy
  *
  function ProcessFile
  *===========================================================================
  * Method.....: ProcessFile
  * Author.....: Mark Austen
  * Date.......: December 26, 2001
  * Notice.....: Copyright (c) 2001 Schooner Software Limited,
  * ...........: All Rights Reserved.
  * Abstract...: Calculate the Hash for the supplied file.
  *===========================================================================
  *
  lparameters ;
    pcFilename
  *
  local ;
    lcDigest, ;
    lnHandle, ;
    QQ___QQ
  *
  lcDigest = This.cDigest
  *
  if not empty( pcFilename ) and file( pcFilename )
    lnHandle = fopen( pcFilename, 12 )
    *
    *** Check to see if the file is in use
    *
    if lnHandle # -1
      fclose( lnHandle )
      CalculateFile( @pcFilename, @lcDigest )
      *
    endif && lnHandle # -1
    *
  endif && not empty( pcFilename ) and file( pcFilename )
  *
  return lcDigest
  *
  *===========================================================================
  * End: ProcessFile
  *===========================================================================
  endfunc && ProcessFile
  *
  procedure Setup
  *===========================================================================
  * Method.....: Setup
  * Author.....: Mark Austen
  * Date.......: February 25, 2002
  * Notice.....: Copyright (c) 2002 Schooner Software Limited,
  * ...........: All Rights Reserved.
  * Abstract...: Setup the parameters for This object.
  *===========================================================================
  *
  lparameters ;
    poParameter
  *
  declare SetHash   in DECHash.dll ;
    integer nLogAction
  *
  declare ResetHash in DECHash.dll
  *
  *** Returns a delimited list of available hash algorithms
  *
  declare integer GetAlgorithmList in DECHash.dll ;
    string @cBuffer
  *
  *** Returns the size of the buffer required to hold the list of available
  *** hash algorithms
  *
  declare integer GetAlgorithmBufferSize    in DECHash.dll
  *
  *** Returns a delimited list of available formats
  *
  declare integer GetFormatList in DECHash.dll ;
    string @cBuffer
  *
  *** Returns the size of the buffer required to hold the list of available
  *** formats
  *
  declare integer GetFormatBufferSize    in DECHash.dll
  *
  *** Sets the algorithm to use for subsequent hashes
  *
  declare integer SetAlgorithm     in DECHash.dll ;
    string @cAlgorithm
  *
  *** Returns the currently set algorithm
  *
  declare integer GetAlgorithm     in DECHash.dll ;
    string @cAlgorithm
  *
  *** Sets the return format
  *
  declare integer SetFormatType    in DECHash.dll ;
    string @cFormat
  *
  *** Returns the current format type
  *
  declare integer SetFormatType    in DECHash.dll ;
    string @cFormat
  *
  *** Returns the message digest size for the currently set algorithm
  *
  declare integer GetDigestSize    in DECHash.dll
  *
  *** Calculates the hash for the specified file
  *
  declare integer CalculateFile    in DECHash.dll ;
    string @cFilename, ;
    string @cBuffer
  *
  *** Calculates the hash for the specified string
  *
  declare integer CalculateString  in DECHash.dll ;
    string @cString, ;
    string @cBuffer
  *
  local ;
    lnBufferSize, ;
    lcBuffer, ;
    lcAlgorithm, ;
    lcFormat, ;
    lnDigestSize, ;
    lcFilename, ;
    QQ___QQ
  *
  store ;
      '' ;
    to ;
      lcBuffer, ;
      lcAlgorithm, ;
      lcFormat, ;
      lcFilename    
  *
  store ;
      0 ;
    to ;
      lnBufferSize, ;
      lnDigestSize
  *
  with This
    SetHash( 0 )
    *
    lnBufferSize = GetAlgorithmBufferSize()
    lcBuffer     = space( lnBufferSize )
    *
    GetAlgorithmList( @lcBuffer )
    .cAlgorithmList = lcBuffer
    *
    *** Set up the formats list
    *
    lnBufferSize = GetFormatBufferSize()
    lcBuffer     = space( lnBufferSize )
    *
    GetFormatList( @lcBuffer )
    .cFormatTypeList = lcBuffer
    *
    if IsObject( .oParameter )
      *
      *** Set the required algorithm
      *
      lcAlgorithm = .oParameter.cAlgorithm
      SetAlgorithm( @lcAlgorithm )
      *
      *** Set the required format
      *
      lcFormat = .oParameter.cFormatType
      SetFormatType( @lcFormat )
      *
    endif && IsObject( .oParameter )
    *
    *** Set up the correct sized buffer for hash result
    *
    lnDigestSize = GetDigestSize()
    .cDigest     = space( lnDigestSize * 2 )
    *
  endwith && This
  *
  return
  *
  *===========================================================================
  * End: Setup
  *===========================================================================
  endproc && Setup
  *
enddefine && MakeHash
*
