define Class ProcessFolders as Custom
  *
  * Member variables
  * ----------------
  *
  oParameter = null
  oHash      = null
  *
  * Public Interface
  * ----------------
  *
  #define FILE_SIZE          2
  *
  procedure Init
  *===========================================================================
  * Method.....: Init
  * Author.....: Mark Austen
  * Date.......: November 15, 2002
  * Notice.....: Copyright (c) 2002 Schooner Software Limited,
  * ...........: All Rights Reserved.
  * Abstract...: Setup the parameters for This object.
  *===========================================================================
  *
  lparameters ;
    poParameter
  *
  with This
    .Setup( poParameter )
    *
  endwith && This
  *
  return
  *
  *===========================================================================
  * End: Init
  *===========================================================================
  endproc && Init
  *
  procedure Destroy
  *===========================================================================
  * Method.....: Destroy
  * Author.....: Mark Austen
  * Date.......: November 15, 2002
  * Notice.....: Copyright (c) 2002 Schooner Software Limited,
  * ...........: All Rights Reserved.
  * Abstract...: Null any objects
  *===========================================================================
  *
  with This
    .oHash.Release()
    *
  endwith && This
  *
  return
  *
  *===========================================================================
  * End: Destroy
  *===========================================================================
  endproc && Destroy
  *
  procedure Release
  *===========================================================================
  * Method.....: Release
  * Author.....: Mark Austen
  * Date.......: November 15, 2002
  * Notice.....: Copyright (c) 2002 Schooner Software Limited,
  * ...........: All Rights Reserved.
  * Abstract...: To bring Class inline with VFP native Release method.
  *===========================================================================
  *
  release This
  *
  return
  *
  *===========================================================================
  * End: Release
  *===========================================================================
  endproc
  *
  procedure Setup
  *===========================================================================
  * Method.....: Setup
  * Author.....: Mark Austen
  * Date.......: November 15, 2002
  * Notice.....: Copyright (c) 2002 Schooner Software Limited,
  * ...........: All Rights Reserved.
  * Abstract...: Setup the parameters for This object.
  *===========================================================================
  *
  lparameters ;
    poParameter
  *
  with This
    .oHash = createobject( 'MakeHash', poParameter )
    *
    if vartype( poParameter ) = O and not IsNull( poParameter )
      .oParameter = poParameter
      *
    endif && IsObject( poParameter )
    *
  endwith && This
  *
  return
  *
  *===========================================================================
  * End: Setup
  *===========================================================================
  endproc && Setup
  *
  procedure Process
  *===========================================================================
  * Method.....: Process
  * Author.....: Mark Austen
  * Date.......: November 15, 2002
  * Notice.....: Copyright (c) 2001 Schooner Software Limited,
  * ...........: All Rights Reserved.
  * Abstract...: Print the supplied string.
  *===========================================================================
  *
  local ;
    lnFoldersUnprocessed, ;
    lnFolders, ;
    lcFolder, ;
    lcPath, ;
    lcSourcePath, ;
    lnI, ;
    lcSourceRoot, ;
    QQ___QQ
  *
  local array ;
    laFolders[ 1 ]
  *
  lcSourceRoot = lower( addbs( .oParameter.cSourcePath ))
  *
  *** Create a cursor to hold the files and hashes
  *
  lnHashLength = len( This.oHash.cDigest )
  *
  create ;
    cursor ;
      'curFiles' ;
      ( ;
        cFolder c( 254 ), ;
        cFile   c( 254 ), ;
        nSize   i, ;
        cHash   c( lnHashLength ) ;
      )
  *
  index on cHash tag cHash
  *
  *** Create a cursor to hold the folders
  *
  create ;
    cursor ;
      'curFolders' ;
      ( ;
        cFolder    c( 254 ), ;
        cProcessed c( 1 ) ;
      )
  *
  *** Add a record for the root folder of the source drive to start the ball
  *** rolling
  *
  insert ;
    into ;
      'curFolders' ;
    values ;
    ( ;
      lcSourceRoot, ;
      'N' ;
    )
  *
  lnFoldersUnprocessed = 1
  *
  *** Work through the folders and create a list of all the folders found
  *
  do LogAction with 'Process Started', .t.
  *
  This.ShowProgress( 'Creating Folder List...' )
  *
  do while lnFoldersUnprocessed # 0
    locate ;
      for ;
        cProcessed = 'N'
    *
    replace ;
      cProcessed with 'Y'
    *
    lcPath    = addbs( alltrim( cFolder ))
    lnFolders = adir( laFolders, lcPath + '*.*', 'D' )
    *
    if lnFolders > 0
      *
      for lnI = 1 to lnFolders
        *
        if 'D' $ laFolders[ lnI, 5 ]
          lcFolder = alltrim( laFolders[ lnI, 1 ] )
          *
          if not inlist( lcFolder, '.', '..' )
            insert ;
              into ;
                'curFolders' ;
              values ;
              ( ;
                lower( lcPath + lcFolder ), ;
                'N' ;
              )
            *
          endif && inlist( lcFolder, '.', '..' )
          *
        endif && 'D' $ laFolders[ lnI, 5 ]
        *
      next && lnI
      *
    endif && lnFolders > 0
    *
    count to lnFoldersUnprocessed for cProcessed = 'N'
    *
  enddo && while lnFoldersUnprocessed # 0
  *
  scan
    lcSourcePath = addbs( alltrim( cFolder ) )
    *
    .MakeHashes( lcSourcePath )
    *
  endscan
  *
  use in 'curFolders'
  *
  select ;
      * ;
    from ;
      'curFiles' ;
    where ;
      cHash in ;
      ( ;
        select ;
            cHash ;
          from ;
            'curFiles' ;
          group by ;
            cHash ;
          having ;
            count( * ) > 1 ;
      ) ;
    into ;
      table 'curDuplicates' ;
    order by ;
      cHash
  *
  use in 'curFiles'
  *
  return
  *
  *===========================================================================
  * End: Process
  *===========================================================================
  endproc && Process
  *
  procedure MakeHashes
  *===========================================================================
  * Procedure..: MakeHashes
  * Author.....: Mark Austen
  * Date.......: November 15, 2002
  * Notice.....: Copyright (c) 2002 Schooner Software Limited,
  * ...........: All Rights Reserved.
  * Abstract...: get the hases for all the files.
  *===========================================================================
  *
  lparameters ;
    pcSourcePath as string
  *
  local ;
    lnFiles, ;
    lcSourceFile, ;
    lcFile, ;
    lcMD5, ;
    lnSize, ;
    QQ___QQ
  *
  local array ;
    laSourceFiles[ 1 ], ;
    laFile[ 1 ]
  *
  lnFiles = adir( laSourceFiles, pcSourcePath + '*.*' )
  *
  if lnFiles > 0
    *
    for lnI = 1 to lnFiles
      lcMD5        = ''
      lcFile       = lower( alltrim( laSourceFiles[ lnI, 1 ] ) )
      lcSourceFile = pcSourcePath + lcFile
      *
      with This
        .ShowProgress( lcSourceFile )
        lcMD5 = .oHash.ProcessFile( lcSourceFile )
        *
      endwith && This
      *
      if not empty( lcMD5 )
        adir( laFile, lcSourceFile )
        lnSize = laFile[ 1, FILE_SIZE ]
        *
        insert ;
          into ;
            'curFiles' ;
            values ;
            ( ;
              pcSourcePath, ;
              lcFile, ;
              lnSize, ;
              lcMD5 ;
            )
        *
      endif && not empty( lcMD5 )
      *
    next && lnI
    *
  endif && lnFiles > 0
  *
  return
  *
  *===========================================================================
  * End: MakeHashes
  *===========================================================================
  endproc && MakeHashes
  *
  procedure ShowProgress
  *===========================================================================
  * Method.....: ShowProgress
  * Author.....: Mark Austen
  * Date.......: November 15, 2002
  * Notice.....: Copyright (c) 2002 Schooner Software Limited,
  * ...........: All Rights Reserved.
  * Abstract...: Dislplay the progress of the Process if required.
  *===========================================================================
  *
  lparameters ;
    pcMessage as string
  *
  local ;
    lcAction, ;
    QQ___QQ
  *
  with This .oParameter
   *
    if not.lQuiet
      *
      if pemstatus( .oParent, ShowProgress, 5 )
        *
        with .oParent
          *
         .ShowProgress( pcMessage )
          *
        endwith && .oParent
        *
      endif && pemstatus( .oParameter.oParent, ShowProgress, 5 )
      *
    endif && not .lQuiet
    *
  endwith && This .oParameter
 *
  return
  *
  *===========================================================================
  * End: ShowProgress
  *===========================================================================
  endproc && ShowProgress
  *
enddefine && ProcessFolders
*
