***********************************************************************
* Program....: ADDPARSE.PRG
* Date.......: 30 November 2002
* Notice.....: Copyright (c) 2002 Tightline Computers, Inc
* Compiler...: Visual FoxPro 08.00.0000.2319 for Windows 
* Purpose....: Class definition for the address parser
***********************************************************************

DEFINE CLASS addressparser AS Session
  DIMENSION aCities[1]
  nMinMatchLength = 4
  
  ********************************************************************
  *** [E] PARSEADDRESS(): Return address object from a multi-line address
  ********************************************************************
  FUNCTION ParseAddress( tcAddress )
  LOCAL ARRAY laSource[1]
  LOCAL loAddrObj, lnLines, lcCity, lcZip, lcState, lnRow
    WITH This
      *** Create the return object
      loAddrObj = .CreateDataObject()
      *** Check that we got something that might be an address
      IF VARTYPE( tcAddress ) # "C" OR EMPTY( tcAddress )
        *** Not a string! The object's success flag is initialized to .T.
        *** So set it to FALSE and return the object right here.
        loAddrObj.lSuccess = .F.
        RETURN loAddrObj
      ENDIF

      *** Get the input string into an array and then ensure that it is clean
      ALINES( laSource, tcAddress, .T. )
      lnLines = .CleanArray( @laSource )
      IF lnLines < 2
        *** We need TWO lines for a valid address, so bale out here too!
        loAddrObj.lSuccess = .F.
        RETURN loAddrObj
      ENDIF
       
      *** If we get this far, we have something that at least looks like it might be an address      
      WITH loAddrObj
        *** Get the first, and (second if present) line of the address from the array
        .cAddr1 = CHRTRAN( laSource[1], ".,;:!()[]{}/\" + CHR(34) + CHR(39), "")
        .cAddr2 = IIF( lnLines > 2, CHRTRAN( laSource[2], ".,;:!()[]{}/\" + CHR(34) + CHR(39), ""), "" )
        *** Next parse out the City from the last line of the array
        lcCity  = ALLTRIM( GETWORDNUM( laSource[ lnLines ], 1,  ','  ))
        *** And the Zip Code
        lcZip   = GETWORDNUM( laSource[ lnLines ], GETWORDCOUNT( laSource[ lnLines ], ', ' ), ', ' )      
        *** Finally we can get the state
        lcState = STRTRAN( laSource[ lnLines ], lcCity, "", 1, 1, 3 )
        lcState = ALLTRIM( CHRTRAN( STRTRAN( lcState, lcZip, "", -1, -1, 3 ), ",", "" ))
        *** And save those to the return object properties too
        .cCity  = lcCity
        .cState = lcState
        .cZip   = lcZip
        *** Now we need to get the best we can for a 5 character ZIP
        lcZip = This.ParseZip( lcZip )
        *** If we got something back, we need to do the lookup,
        *** Otherwise we are finished
        IF NOT EMPTY( lcZip )
          *** Store the Zip Code we used to look up
          .cLUZip = lcZip
          *** Now try the look up
          lnCities = This.ZipLookUp( lcZip )
          DO CASE
            CASE lnCities = 1
              *** We got one and one result only 
              lcState = GETWORDNUM( This.aCities[1], GETWORDCOUNT( This.aCities[1], ' ' ), ' ')
              *** The state is always "_XX" at the end of the line
              lcCity  = LEFT( This.aCities[1], LEN( This.aCities[1]) - 3 )
            CASE lnCities > 1
              *** We got multiple results, get the row which has the best match to the 
              *** City that we got from parsing the original data.
              lnRow = This.FindMatch( lcCity )
              IF lnRow > 0
                *** The best match was in the specified row
                lcState = GETWORDNUM( This.aCities[ lnRow ], GETWORDCOUNT( This.aCities[ lnRow ], ' ' ), ' ')
                *** The state is always "_XX" at the end of the line
                lcCity  = LEFT( This.aCities[ lnRow ], LEN( This.aCities[ lnRow ]) - 3 )
              ELSE
                *** We couldn't find a good match, so do nothing more
                lcState = ""
                lcCity  = ""
              ENDIF  
            OTHERWISE
              *** We didn't get a match, so do nothing more
              lcState = ""
              lcCity  = ""
          ENDCASE
          .cLUCity  = lcCity
          .cLUState = lcState
        ENDIF 
      ENDWITH
    ENDWITH
    *** Return the object 
    RETURN loAddrObj
  ENDFUNC

  ********************************************************************
  *** [P] FINDMATCH(): Return the row in the aCities Array that most closely matches the passed in city name
  ********************************************************************
  PROTECTED FUNCTION FIndMatch( tcCity )
    LOCAL lnMinLength, lnRow, lcCity
    WITH This
      lnMinLength = This.nMinMatchLength 
      lcCity = ALLTRIM( tcCity )
      lnRow = 0

      *** Keep looking until we either find a match or the length of the
      *** city gets too short
      DO WHILE LEN( lcCity ) => lnMinLength AND lnRow = 0
        *** Case Insensitive search; Return row number; Exact OFF
        lnRow = ASCAN( .aCities, lcCity, -1, -1, 1, 13 )
        *** If we do not have a match yet, shorten the city
        IF lnRow = 0
          lcCity = LEFT( lcCity, LEN( lcCity ) - 1 )
        ENDIF
      ENDDO

      *** If we still don't have a match, see if the passed city
      *** contains multiple words
      IF lnRow = 0
        IF GETWORDCOUNT( tcCity, ' ' ) > 1
          *** It has multiple words, so check to see if the second word
          *** is included in an array entry; i.e., Ft. Lauderdale entered as the
          *** city in the memo field but Fort Lauderdale returned as the city in
          *** the array
          lcCity = GETWORDNUM( tcCity, 2, ' ' ) 
          FOR lnCnt = 1 TO ALEN( .aCities )
            *** Since each array element contains both city and state information
            *** We have to check for a word count of 3 in the array element
            IF GETWORDCOUNT( .aCities[ lnCnt ], ' ' ) > 2
              IF ATC( lcCity, .aCities[ lnCnt ] ) > 0
                *** Yep. Got one
                lnRow = lnCnt
                EXIT
              ENDIF
            ENDIF
          ENDFOR
        ENDIF
      ENDIF
    ENDWITH
    *** Return the row number of the array that contains the
    *** match or 0 if no match was found
    RETURN lnRow      
  ENDFUNC

  ********************************************************************
  *** [P] ZIPLOOKUP(): Call the Web Service Lookup
  ********************************************************************
  PROTECTED FUNCTION ZipLookUp( tcZip )
    LOCAL ARRAY laRes[1]
    LOCAL loException, lcErrorMsg, loWSHandler, loZipcodeLookupService, lnRetVal
    TRY
      **********************************************************************************
      loWSHandler = NEWOBJECT("WSHandler",IIF(VERSION(2)=0,"",HOME()+"FFC\")+"_ws3client.vcx")
      loZipcodeLookupService = loWSHandler.SetupClient("http://HOSTING.MSUGS.CH/cheeso9/zips/ZipService.asmx?WSDL", "ZipcodeLookupService", "ZipcodeLookupServiceSoap")
      laRes = loZipcodeLookupService.ZipToCityAndState( tcZip )
      IF ALEN( laRes, 1 ) > 0 AND NOT EMPTY( NVL( laRes[1], "") )
        *** We got something back
        DIMENSION This.aCities[1]
        ACOPY( laRes, This.aCities )
        lnRetVal = ALEN( laRes, 1 )
      ELSE
        lnRetVal = 0
      ENDIF
      **********************************************************************************
      CATCH TO loException
        lcErrorMsg= "Error: " + TRANSFORM(loException.Errorno) + " - " + loException.Message
        DO CASE
          CASE VARTYPE( loZipcodeLookupService ) # "O"
            *** Handle error creating web service
            lcErrorMsg = "Unable to connect to Web Service" + CHR(13) + lcErrMsg
          CASE !EMPTY( loZipcodeLookupService.FaultCode )
            *** Handle 
            lcErrorMsg = loZipcodeLookupService.Detail + CHR(13) + lcErrMsg
          OTHERWISE
            *** Just display the error message we got
        ENDCASE
        * Use for debugging purposes
        MESSAGEBOX(lcErrorMsg)
      **********************************************************************************
      FINALLY
        *** Make sure that if something went wrong, we return a NULL result
        IF VARTYPE( lnRetVal ) # "N"
          lnRetVal = 0
        ENDIF  
      **********************************************************************************
    ENDTRY
    RETURN lnRetVal
  ENDFUNC

  ********************************************************************
  *** [P] PARSEZIP(): Return 5 digit Zip or empty string from a passed in Zip Code
  ********************************************************************
  PROTECTED FUNCTION ParseZip( tcZip )
    LOCAL lcZip, lcRetVal
    *** We need FoxTools for this one
    IF NOT "FOXTOOLS" $ SET("Library")
      SET LIBRARY TO (HOME() + "FOXTOOLS.FLL") ADDITIVE
    ENDIF
    *** Filter out all but numbers
    lcZip = StrFilter( tcZip, "0123456789")
    *** All valid will Zips have at least 5 characters
    lcRetVal = IIF( LEN( lcZip ) >= 5, LEFT( lcZip, 5 ), "" )
    *** Return whatever we got
    RETURN lcRetVal  
  ENDFUNC

  ********************************************************************
  *** [P] CLEANARRAY(): Remove blank lines from athe array passed by Reference
  *** ................: Returns actual number of lines in the resulting array
  ********************************************************************
  PROTECTED FUNCTION CleanArray( taArray )
    LOCAL lnLines, lnNextLine, lnProcessed, lnValidLines
    *** Get The number of rows currently in the Array
    lnLines = ALEN( taArray, 1 )
    STORE 1 TO lnNextLine
    STORE 0 TO lnProcessed, lnValidLines
    DO WHILE .T.
      *** Delete blank lines
      IF EMPTY( taArray[ lnNextLine ] )
        *** Move the blank line to the bottom of array, 
        *** So we don't increment the line counter here
        ADEL( taArray, lnNextLine )
      ELSE 
        *** This is a valid line, so increment the line counter    
        lnNextLine = lnNextLine + 1 
        *** And the number of non=empty lines
        lnValidLines = lnValidLines + 1 
      ENDIF
      *** Increment the Process Counter and exit when we have processed all lines
      lnProcessed = lnProcessed + 1 
      IF lnProcessed = lnLines
        EXIT
      ENDIF
    ENDDO      
    *** Re-Dimension the array to the correct number of rows
    DIMENSION taArray[ lnValidLines ]
    *** Return the number of rows
    RETURN (ALEN( taArray, 1 ))
  ENDFUNC

  ********************************************************************
  *** [P] CREATEDATAOBJECT(): Returns a Data Object with required properties
  ********************************************************************
  PROTECTED FUNCTION CreateDataObject()
    LOCAL loData
    *** Use the new (VFP 8.0) Empty Base class for the Data Object  
    loData = NEWOBJECT( "EMPTY" )
    *** Add properties using the new ADDPROPERTY() function
    *** And initialize them all to empty string
    ADDPROPERTY( loData, 'cAddr1', "" )
    ADDPROPERTY( loData, 'cAddr2', "" )
    ADDPROPERTY( loData, 'cZip', "" )
    ADDPROPERTY( loData, 'cCity', "" )
    ADDPROPERTY( loData, 'cState', "" )
    ADDPROPERTY( loData, 'cLUZip', "" )
    ADDPROPERTY( loData, 'cLUCity', "" )
    ADDPROPERTY( loData, 'cLUState', "" )
    ADDPROPERTY( loData, 'lSuccess', .T. )
    *** Return the initialized object
    RETURN loData    
  ENDFUNC
ENDDEFINE   